#include "/util/common.glsl"

#version 130

varying vec2 texcoord;

#ifdef FSH
    uniform sampler2D gcolor;
    uniform sampler2D shadowcolor0;
    uniform sampler2D shadowtex0;
    uniform sampler2D shadowtex1;
    uniform sampler2D depthtex0;
    uniform sampler2D depthtex1;
    uniform int isEyeInWater;
    uniform mat4 shadowModelView;
    uniform mat4 shadowProjection;
    uniform vec3 shadowLightPosition;

    uniform int worldTime;

    uniform float far;

    uniform mat4x4 gbufferProjection;
    uniform mat4x4 gbufferProjectionInverse;

    vec4 getShadowPos(vec4 playerPos) {
        vec4 shadowPos = (shadowModelView * playerPos);
        shadowPos = shadowProjection * shadowPos;
        float w = 1.;
        if(clamp(shadowPos.xyz*.5+.5,0.,1.)==shadowPos.xyz*.5+.5){}else{w=-1.;}

        float numerator = length(shadowPos.xy)+.1;
    	numerator*=numerator;

        float bias = 1./shadowMapResolution*numerator/.1;
        float factor = length(shadowPos.xy) + 0.1;

        shadowPos.xyz = vec3(shadowPos.xy/factor,shadowPos.z*.5);
        shadowPos.xyz = shadowPos.xyz*.5+.5;
        shadowPos.z -= bias;
        shadowPos.w = w;
        return shadowPos;
}

    void main() {
    	vec3 color = texture2D(gcolor, texcoord).rgb;
        float isTerrain = texture2D(depthtex0, texcoord).a;

        vec2 z = vec2(texture2D(depthtex0, texcoord).r, texture2D(depthtex1, texcoord).r);

        vec4 screenPos = vec4(texcoord, z.x, 1.0);
        vec4 viewPos = gbufferProjectionInverse * (screenPos * 2.0 - 1.0);
        viewPos.xyz/=viewPos.w;

        #ifdef VOLUMETRICS_ENABLED

            vec4 shadowPos;

            vec3 rd = normalize(viewPos.xyz);

            float vd = 0.0;
            vec4 wp = vec4(0,0,0,1);

            vec3 add = vec3(0.);

            float depth = length(viewPos.xyz);

            vec4 c0 = gbufferModelViewInverse * wp;
            vec4 c1 = gbufferModelViewInverse * (wp + vec4(depth*rd, 0.));
            vec4 sp0 = getShadowPos(c0);
            vec4 sp1 = getShadowPos(c1);
            c0.xyz += cameraPosition;
            c1.xyz += cameraPosition;

            float castDist = 20./VOLUMETRIC_STEPS;
            #ifdef CREPUSCULAR_FOG
                vec4 fogColor = vec4(0.75, 0.75, 0.75, 0.02) * (isEyeInWater<1?(pow(float(worldTime)/24000.-1.,6.)*0.75+0.25):1.);
            #else
                vec4 fogColor = vec4(0.75, 0.75, 0.75, 0.02);
            #endif
            if(isEyeInWater==1){fogColor.a=0.20;}
            for(float i=0.; i<=1.0; i+=1. / VOLUMETRIC_STEPS) {

                vec4 p = mix(sp0, sp1, i);
                vec4 c = mix(c0, c1, i);

                vec4 shadow = texture2D(shadowtex1, p.xy);

                if(!(shadow.r < p.z) && sp1.w>=0.) {

                    vec4 tint = vec4(1);
                    if(isEyeInWater>0){
                        color.rgb = mix(color.rgb, fogColor.rgb*vec3(0.23, 0.32, 0.62)*(smoothstep(max(c.y, 50.), 40., 51.)*.5+.5), clamp(fogColor.a * VOLUMETRIC_DENSITY * castDist, 0.0, 1.0));
                    } else {
                        if(!(sp1.w<0.)){
                            #ifdef COLORED_SHADOWS
                                if(texture2D(shadowtex0, p.xy).r < p.z) {
                                    tint = texture2D(shadowcolor0, p.xy);
                                    if(tint.a>0.){
                                        tint.rgb = mix(vec3(1), tint.rgb, tint.a);
                                    }
                                }
                                color.rgb = mix(color.rgb, fogColor.rgb*tint.rgb, clamp(fogColor.a * VOLUMETRIC_DENSITY * castDist, 0.0, 1.0));
                    
                            #else 

                               color.rgb = mix(color.rgb, fogColor.rgb*tint.rgb, clamp(fogColor.a * VOLUMETRIC_DENSITY * castDist, 0.0, 1.0));
                    
                            #endif
                        }else{

                            color.rgb = mix(color.rgb, fogColor.rgb*tint.rgb, clamp(fogColor.a * VOLUMETRIC_DENSITY * castDist, 0.0, 1.0));

                        }
                    }
                }

                

            }

        #else
            if(isEyeInWater==1){color.rgb=mix(color.rgb,vec3(0.23, 0.32, 0.62),0.5);}
        #endif

        if(isEyeInWater==2){color.rgb=mix(color.rgb,vec3(1,0.5,0.0),0.9);}
        if(isEyeInWater==3){color.rgb=mix(color.rgb,vec3(1),0.9);}

        /* DRAWBUFFERS:0 */
    	gl_FragData[0] = vec4(vec3(
            color//texture2D(shadowcolor0, texcoord).rgb
        ), 1.0);
    }
#endif

#ifdef VSH
    void main() {
    	gl_Position = ftransform();
    	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    }
#endif