#version 130

#include "/util/common.glsl"

/*
 * Varyings
 */

float iausfndsjddd = VOLUMETRIC_STEPS;
float unjasfgdujhs = VOLUMETRIC_DENSITY;

varying vec4 glcolor;
varying float depth;

varying vec4 screenSpacePosition;

varying vec2 texcoord;
varying vec2 lmcoord;

varying vec3 normal;

#ifdef PBR
    varying vec3 glPos;
    varying mat3x3 tbn_Matrix;
#endif

#ifdef SHADOWS
    varying float backface;
    //varying vec4 shadowPos;
#endif

/*
 * Uniforms & Attibutes
 */

#ifdef FSH

    //uniform vec3 cameraPosition;
    uniform sampler2D lightmap;
    uniform sampler2D texture;

    uniform float far;

    #ifdef PBR
        uniform sampler2D normals;
        uniform sampler2D specular;
        uniform vec3 skyColor;
    #endif

    #ifdef SHADOWS
        #ifdef COLORED_SHADOWS
            uniform sampler2D shadowcolor0;
        #endif

        uniform sampler2D shadowtex0;
        uniform sampler2D shadowtex1;

        uniform mat4 shadowModelView;
        uniform mat4 shadowProjection;
        uniform vec3 shadowLightPosition;

        uniform vec3 sunPosition;
    #endif
    const bool shadowcolor0Nearest = false;
    const bool shadowtex0Nearest = false;
    const bool shadowtex1Nearest = false;

#endif

#ifdef VSH

    #ifdef SHADOWS
        uniform mat4 shadowModelView;
        uniform mat4 shadowProjection;
        uniform vec3 shadowLightPosition;
    #endif

    attribute vec4 at_tangent;
    attribute vec3 mc_Entity;

#endif
/*
 * Functions
 */

vec3 worldSpace(vec3 map, mat3x3 tbn) {
    return normalize(tbn * map);
}

float F_Schlick(float f0, float theta) {
    return f0 + (1.0 - f0) * pow(1.0 - theta, 5.0);
}

#ifdef FSH

vec4 getShadowPos(vec4 playerPos) {
    vec4 shadowPos = shadowProjection * (shadowModelView * playerPos);

    float numerator = length(shadowPos.xy)+.1;
	numerator*=numerator;

    float bias = 1./shadowMapResolution*numerator/.1;
    float factor = length(shadowPos.xy) + 0.1;

    shadowPos.xyz = vec3(shadowPos.xy/factor,shadowPos.z*.5);
    shadowPos.xyz = shadowPos.xyz*.5+.5;
    shadowPos.z -= bias / abs(backface);
    return shadowPos;
}

void main() {

    vec4 color = texture2D(texture, texcoord) * glcolor;
    #ifndef ENTITIES
    vec2 lmc = lmcoord;
    lmc.y = abs(lmc.y);

    vec3 rd = normalize(glPos);

    #ifdef PBR 
        vec4 normalMap = texture2D(normals, texcoord)*2.-1.;
        //lmc.y *= normalMap.z;
        normalMap.z = sqrt(max(1.0 - dot(normalMap.xy, normalMap.xy), 0.0));
        vec3 ssNormal = worldSpace(normalMap.xyz, tbn_Matrix);
    #else
        vec3 ssNormal = normal;
    #endif

    vec4 playerPosition = gbufferModelViewInverse * screenSpacePosition;
    
    #ifdef SHADOWS

        vec4 shadowPos = getShadowPos(playerPosition);

        float shadow = 1.0;

        float shadowDot = clamp(dot(ssNormal, normalize(shadowLightPosition)), AMBIENT, 1.0);
        float isDay = dot(shadowLightPosition, sunPosition);
        shadowPos.w = dot(ssNormal, normalize(shadowLightPosition));
        if(shadowPos.w>0.01) {

            
            if(texture2D(
                #ifdef COLORED_SHADOWS
                    shadowtex1,
                #else
                    shadowtex0,
                #endif
            shadowPos.xy).r < shadowPos.z) {

                #ifdef WATER
                    shadow *= clamp(AMBIENT * 1.7, 0.0, 1.0);
                #else
                    shadow *= AMBIENT;
                #endif

            } else {
                #ifdef PBR
                    float theta = dot(ssNormal, -rd);

			        vec4 spec = texture2D(specular, texcoord);
			        float sheen = F_Schlick(spec.g, theta);

			        vec3 sdir = normalize(shadowLightPosition);
			        vec3 rdir = reflect(rd, ssNormal);

			        vec3 skyCol = skyColor;
			        vec3 sunCol = pow(clamp(dot(rdir, sdir), 0.,1.), 10.)*vec3(1.0, 0.9, 0.9);

                    vec3 shine = ((1.-spec.r)*skyCol*.2 + spec.r*(+sunCol))*sheen;

			        color.rgb += shine;

                    #ifdef WATER

                        color.a += clamp(dot(shine, vec3(.5,.5,.5)),0.,1.);

                    #endif
                #endif
                #ifdef COLORED_SHADOWS
                    vec4 tint = vec4(1);
                    if(texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
                        tint = texture2D(shadowcolor0, shadowPos.xy);
                        if(tint.a>=0.){
                            tint.rgb = mix(vec3(1), tint.rgb, tint.a);
                            tint.rgb = mix(tint.rgb, vec3(1), lmc.x);
                        }
                    }
                    color.rgb *= tint.rgb;
                #endif

            }

            shadowDot = sqrt(shadowDot);

        } else {

            #ifndef PBR

                shadow *= AMBIENT;

            #endif

        }
        #ifdef PBR

            shadow *= shadowDot;

        #endif
        
        color.rgb *= shadow;
    #endif
    color *= texture2D(lightmap, lmc);
    #endif
    /* DRAWBUFFERS:0 */
	gl_FragData[0] = color;
    

}

#endif

#ifdef VSH

void main() {

    glcolor = gl_Color;

    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
	lmcoord  = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    normal = normalize(gl_NormalMatrix * gl_Normal);

    vec4 vertex = gl_Vertex;

    screenSpacePosition = gl_ModelViewMatrix * vertex;
    vec4 playerPosition = gbufferModelViewInverse * screenSpacePosition;

    depth = length(playerPosition);

    #ifdef WAVING

        int blocktype = int(mc_Entity.x+0.5);
        if(blocktype==10003){
            screenSpacePosition = foliage_Wave(screenSpacePosition, 1.0);
        }
        if(blocktype==10002){
            screenSpacePosition = vec4(foliage_Wave(screenSpacePosition, 1.5).xzw, screenSpacePosition.y).xwyz;
        }
        if(blocktype==10000){
            screenSpacePosition = water_Wave(screenSpacePosition);
        }

    #endif

    glPos = screenSpacePosition.xyz;

    #ifdef PBR
        vec3 binormal = normalize(gl_NormalMatrix * cross(at_tangent.xyz, gl_Normal.xyz) * at_tangent.w);
	    vec3 tangent  = normalize(gl_NormalMatrix * at_tangent.xyz);

	    tbn_Matrix = mat3x3(tangent, binormal, normal);
    #endif
    backface = dot(normalize(shadowLightPosition), normal);

    gl_Position = gl_ProjectionMatrix * screenSpacePosition;
    
}

#endif