const float shadowDistance = 324.0;
const float sunPathRotation = 30.0;

#define SHADOWS

#define AMBIENT 0.75 //[0.25 0.5 0.75]
#define COLORED_SHADOWS

const int shadowMapResolution = 1024; //[512 1024 2048 4096 8192]

#define PBR

#define WAVING
#define WAVING_SPEED 1.5 // [0.25 0.5 0.75 1.0 1.25 1.5 1.75 2.0]
#define WAVING_STRENGTH 1.0 // [0.5 0.75 1.0 1.25 1.5]

#define VOLUMETRIC_STEPS 20.0 // [10.0 20.0 30.0 40.0 50.0 60.0 70.0 80.0 90.0 100.0]
#define VOLUMETRIC_DENSITY 1.0 // [0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0 1.1 1.2 1.3 1.4 1.5 1.6 1.7 1.8 1.9 2.0]
#define VOLUMETRICS_ENABLED
#define CREPUSCULAR_FOG

uniform float frameTimeCounter;
uniform vec3 cameraPosition;
uniform mat4x4 gbufferModelViewInverse;
uniform mat4x4 gbufferModelView;

float sinp(float x) {
    return (sin(2.*x)+sin(3.141592653589793238*x))/2.;
}

vec4 foliage_Wave(vec4 screenSpacePosition, float str) {

    vec4 playerPosition = gbufferModelViewInverse * screenSpacePosition;
    vec3 worldPos = playerPosition.xyz + cameraPosition.xyz;
    worldPos *= vec3(0.5, 0.25, 0.5);

    float wind = frameTimeCounter * 170. * WAVING_SPEED;

    float magnitude = sin(wind * 0.0027 + worldPos.z + worldPos.y) * 0.04 + 0.04;
    magnitude *= str;
    float d0 = sin(wind * 0.0093);
    float d1 = sin(wind * 0.0129);
    float d2 = sin(wind * 0.0102);
    vec3 wave;
    wave.x = sin(wind*0.0063 + d0 + d1 - worldPos.x + worldPos.z + worldPos.y) * magnitude;
    wave.z = sin(wind*0.0224 + d1 + d2 + worldPos.x - worldPos.z + worldPos.y) * magnitude;
    wave.y = sin(wind*0.0015 + d2 + d0 + worldPos.z + worldPos.y - worldPos.y) * magnitude;

    wave *= 0.1;
    wave *= vec3(8.0, 3.0, 4.0);

    playerPosition.xyz += wave;

    return gbufferModelView * playerPosition;

}

vec4 water_Wave(vec4 screenSpacePosition) {

    vec4 playerPosition = gbufferModelViewInverse * screenSpacePosition;
    vec3 worldPos = playerPosition.xyz + cameraPosition.xyz;
    worldPos *= vec3(0.5, 0.25, 0.5);

    float timeC = frameTimeCounter * WAVING_SPEED;

    float wave = sinp(worldPos.x+timeC)*sinp(worldPos.z-timeC);

    playerPosition.y += wave*.15*1./(length(screenSpacePosition.xyz)*.2+1.)*WAVING_STRENGTH;

    return gbufferModelView * playerPosition;

}